ingame = gamestate.new()

-- Handles entities
require 'entities'

-- HUMP Camera
camera = require 'lib.camera'


function ingame:init()
	-- Initilizes the entitiy system
	entities.init()
	
	-- Create the camera at 100,100 | zoom of 2
	cam = camera(100, 100, 4)
	
	-- The mouse selection rectangle dimensions
	selection = {
		sx = 0,
		sy = 0,
		ex = 0,
		ey = 0
	}
	-- Which team the player can control
	playerTeam = 1
	
	-- Create a sample entity, box
	entities.create('box', 128,128, 1)
end

function ingame:enter(previous)
	currentGamestate = 'ingame'
	
	-- Draw green background
	love.graphics.setBackgroundColor(75, 150, 50)
end

function ingame:leave()
end


function ingame:update(dt)
	-- Entity Selection
	if love.mouse.isDown('l') then
		local cx,cy = cam:mousepos()
		setSelectionEnd(cx,cy)
		entities.deselectAll()
	else
		selection.sx,selection.sy, selection.ex,selection.ey = 0,0, 0,0
	end
	entities.select(selection.sx,selection.sy, selection.ex,selection.ey)
	
	-- Camera Controls
	if love.keyboard.isDown('w') then
		cam:move(0, -dt*100)
	elseif love.keyboard.isDown('s') then
		cam:move(0, dt*100)
	end
	if love.keyboard.isDown('a') then
		cam:move(-dt*100, 0)
	elseif love.keyboard.isDown('d') then
		cam:move(dt*100, 0)
	end
	
	-- Update Entities
	entities.update(dt)
	bump.collide()
end

function ingame:draw()
	-- Set lines white and pixel-perfect
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setLine(1, 'smooth')
		
	-- Draw selection indicator behind entities
	entities.drawSelectionIndicators()
	
	cam:attach() -- Begin drawing through the camera
	-- Draw the Entities
	entities.draw()
	
	-- Draw Health bars in front of entities
	entities.drawHealthIndicators()
	entities.drawLOS()
	cam:detach() -- Stop drawing through the camera
	
	-- Draw the mouse selection rectangle
	local sx,sy = cam:cameraCoords(selection.sx,selection.sy)
	local ex,ey = cam:cameraCoords(selection.ex,selection.ey)
	love.graphics.setColor(255, 255, 255, 25)
	love.graphics.rectangle('fill', sx,sy, ex-sx,ey-sy)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.rectangle('line', sx,sy, ex-sx,ey-sy)
end

function ingame:focus(bool)
end

function ingame:keypressed(key, unicode)
	if key == 'escape' then
		gamestate.switch(menu)
	end
	
	-- Spawns a new team 1 entity at mouse
	if key == '1' then
		local cx,cy = cam:mousepos()
		entities.create('box', cx, cy, 1)
	end
	-- Spawns a new team 2 entity at mouse
	if key == '2' then
		local cx,cy = cam:mousepos()
		entities.create('box', cx, cy, 2)
	end
	
	-- Spawns a new team 1 entity at mouse
	if key == '3' then
		for i=1,30 do
			local cx,cy = cam:mousepos()
			entities.create('box', cx, cy, 1)
		end
	end
	-- Spawns a new team 2 entity at mouse
	if key == '4' then
		for i=1,30 do
			local cx,cy = cam:mousepos()
			entities.create('box', cx, cy, 2)
		end
	end
	
	-- Switches to menu gamestate
	if key == 'm' then
		gamestate.switch(menu)
	end
	
	-- Destroy all entities
	if key == 'r' then
		entities.destroyAll()
	end
	
	-- Zoom camera
	if key == 'e' then
		if cam.zoom < 16 then
			cam.zoom = cam.zoom * 2
		end
	end
	if key == 'q' then
		if cam.zoom > 1 then
			cam.zoom = cam.zoom / 2
		end
	end
end

function ingame:keyreleased(key, unicode)
end

function ingame:mousepressed(x,y, button)
	local cx,cy = cam:mousepos()
	cx,cy = math.floor(cx),math.floor(cy)
	
	-- Starts selecting entities
	if button == 'l' then
		setSelectionStart(cx,cy)
	end
	
	-- Sets target for entities
	if button == 'r' then
		entities.setTargetPos(cx,cy)
	end
end

function ingame:mousereleased(x,y, button)
end

function ingame:quit()
end

-- TODO SEE IF OPTIMIZABLE
-- Determines if entities is inside selection rectangle
function insideSelection(ssx,ssy, sex,sey, bsx,bsy, bex,bey)
	if ssx < bsx and sex > bsx or ssx > bsx + bex and sex < bsx + bex or ssx > bsx and ssx < bsx + bex then
		if ssy < bsy and sey > bsy or ssy > bsy + bey and sey < bsy + bey or ssy > bsy and ssy < bsy + bey then
			return true
		end
	end
	return false
end

-- Sets start corner of selection rectangle
function setSelectionStart(x,y)
	selection.sx = x
	selection.sy = y
end

-- Sets end corner of selection rectangle
function setSelectionEnd(x,y)
	selection.ex = x
	selection.ey = y
end
