--[[
Copyright (c) 2012 Matthias Richter

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Except as contained in this notice, the name(s) of the above copyright holders
shall not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
]]--

-- default style
local color = { --  Box Color            Bottom Box Color     Text Color          Bottom Text Color     Border Color
	normal = {bg = {220,220,220}, bgb = {170,170,170}, fg = { 50, 50, 50}, fgb = {230,230,230}, border={ 50, 50, 50}},
	hot    = {bg = {220,220,220}, bgb = {170,170,170}, fg = { 50, 50, 50}, fgb = {230,230,230}, border={255,255,255}},
	active = {bg = {160,160,160}, bgb = {110,110,110}, fg = {230,230,230}, fgb = { 50, 50, 50}, border={250,250,250}}
}

local function box(x,y,w,h, bg, bgb, border, state)
	love.graphics.setLine(2, 'rough')
	
	if state ~= 'active' then
		love.graphics.setColor(bgb)
		love.graphics.rectangle('fill', x,y,w,h)
		love.graphics.setColor(bg)
		love.graphics.rectangle('fill', x,y,w,h-10)
		love.graphics.setColor(border)
		love.graphics.rectangle('line', x,y,w,h)
	else
		love.graphics.setColor(bgb)
		love.graphics.rectangle('fill', x,y+5,w,h-5)
		love.graphics.setColor(bg)
		love.graphics.rectangle('fill', x,y+5,w,h-10)
		love.graphics.setColor(border)
		love.graphics.rectangle('line', x,y+5,w,h-5)
	end
end

-- load default font
if not love.graphics.getFont() then
	love.graphics.setFont(love.graphics.newFont(12))
end

local function Button(state, title, x,y,w,h)
	local c = color[state]
	box(x,y,w,h, c.bg, c.bgb, c.border, state)
	local f = assert(love.graphics.getFont())
	x,y = x + (w-f:getWidth(title))/2, y - 5 + (h-f:getHeight(title))/2
	
	if state ~= 'active' then
		love.graphics.setColor(c.fgb)
		love.graphics.print(title, x,y+1)
		love.graphics.setColor(c.fg)
		love.graphics.print(title, x,y)
	else
		love.graphics.setColor(c.fgb)
		love.graphics.print(title, x,y+6)
		love.graphics.setColor(c.fg)
		love.graphics.print(title, x,y+5)
	end
end

local function Label(state, text, align, x,y,w,h)
	local c = color[state]
	love.graphics.setColor(c.fgb)
	local f = assert(love.graphics.getFont())
	y = y + (h - f:getHeight(text))/2
	if align == 'center' then
		x = x + (w - f:getWidth(text))/2
	elseif align == 'right' then
		x = x + w - f:getWidth(text)
	end
	love.graphics.print(text, x,y)
end

local function Slider(state, fraction, vertical, x,y,w,h)
	local c = color[state]
	
	love.graphics.setLine(2, 'rough')
	love.graphics.setColor(c.bgb)
	love.graphics.rectangle('fill', x-10,y+h/4, w+10,h/2)
	love.graphics.setColor(c.border)
	love.graphics.rectangle('line', x-10,y+h/4, w+10,h/2)

	if vertical then
		y = math.floor(y + h * fraction - 10)
		h = 10
	else
		x = math.floor(x + w * fraction - 10)
		w = 10
	end
	box(x,y,w,h-10, c.bg,c.bgb, c.border)
end

local function Slider2D(state, fraction, x,y,w,h)
	local c = color[state]
	box(x,y,w,h, c.bg,c.bgb, c.border)

	-- draw quadrants
	love.graphics.setLine(1, 'rough')
	love.graphics.setColor(c.fg[1], c.fg[2], c.fg[3], math.min(127,c.fg[4] or 255))
	love.graphics.line(x+w/2,y, x+w/2,y+h)
	love.graphics.line(x,y+h/2, x+w,y+h/2)

	-- draw cursor
	local xx = math.ceil(x + fraction[1] * w)
	local yy = math.ceil(y + fraction[2] * h)
	love.graphics.setColor(c.fg)
	love.graphics.line(xx-3,yy,xx+2,yy)
	love.graphics.line(xx,yy-3,xx,yy+2)
end

local function Input(state, text, cursor, x,y,w,h)
	local c = color[state]
	box(x-10,y,w,h, c.bg,c.bgb, c.border, state ~= 'active')

	local f = love.graphics.getFont()
	local th = f:getHeight(text)
	local cursorPos = x + 5 + f:getWidth(text:sub(1,cursor))
	
	love.graphics.setColor(color.normal.fgb)
	love.graphics.print(text, x-3,y-4+(h-th)/2)
	love.graphics.setColor(color.normal.fg)
	love.graphics.print(text, x-3,y-5+(h-th)/2)
	
	if state == 'hot' then
		love.graphics.setLine(2, 'rough')
		love.graphics.setColor(color.normal.border)
		love.graphics.line(cursorPos-6, y+4, cursorPos-6, y+h-14)
	end
end

local function Checkbox(state, checked, label, align, x,y,w,h)
	local c = color[state]
	local bw, bx, by  = math.min(w,h)*.7, x, y
	by = y + (h-bw)/2

	local f = assert(love.graphics.getFont())
	local tw,th = f:getWidth(label), f:getHeight(label)
	local tx, ty = x, y + (h-th)/2
	if align == 'left' then
		-- [ ] LABEL
		bx, tx = x, x+bw+4
	else
		-- LABEL [ ]
		tx, bx = x, x+4+tw
	end

	if checked or state == 'active' then
		box(bx,by,bw,bw, color.active.bg,color.active.bgb, c.border, 'active')
	else
		box(bx,by,bw,bw, c.bg,c.bgb, c.border)
	end

	love.graphics.setColor(color.normal.fgb)
	love.graphics.print(label, tx, ty)
end


-- the style
return {
	color    = color,
	gradient = gradient,

	Button   = Button,
	Label    = Label,
	Slider   = Slider,
	Slider2D = Slider2D,
	Input    = Input,
	Checkbox = Checkbox,
}
