local unit = {
	x = 0,
	y = 0,
	w = 16,
	h = 16,
	
	health = 10,
	maxHealth = 10,
	speed = 10,
	sight = 10,
	damage = 1,
	
	selected = false,
	team = 1,
	
	image = love.graphics.newImage('assets/naked.png'),
	
	tx = 0,
	ty = 0,
	targetEntity = unit
}

function unit:setPos(x,y)
	if unit.x == x and unit.y == y then
		unit.tx = x
		unit.ty = y
	else
		unit.x = x
		unit.y = y
	end
end

function unit:getPos()
	return unit.x, unit.y
end


function unit:setSize(w,h)
	unit.w = w
	unit.h = h
end

function unit:getSize()
	return unit.w, unit.h
end


function unit:setSpeed(speed)
	unit.speed = speed
end

function unit:getSpeed()
	return unit.speed
end


function unit:setSight(range)
	unit.sight = range
end

function unit:getSight()
	return unit.sight
end


function unit:setTeam(team)
	unit.team = team
end

function unit:getTeam()
	return unit.team
end


function unit:select()
	unit.selected = true
end

function unit:deselect()
	unit.selected = false
end

function unit:isSelected()
	return unit.selected
end


function unit:setTargetPos(x,y)
	unit.tx = x
	unit.ty = y
end

function unit:getTargetPos()
	return unit.tx, unit.ty
end

function unit:setTargetEntity(target)
	unit:setTargetPos(target:getPos())
end

function unit:getTargetEntity()
	return unit.targetEntity
end


function unit:setHealth(health, maxHealth)
	unit.health,unit.maxHealth = health or unit.health, maxHealth or health
	if unit.health > unit.maxHealth then
		unit.health = unit.maxHealth
	end
end

function unit:getHealth()
	return unit.health, unit.maxHealth
end


function unit:moveToTarget(dt)
	if unit:getPos() ~= unit:getTargetPos() then
		local x,y = unit:getPos()
		local tx,ty = unit:getTargetPos()
		local w,h = unit:getSize()
		local speed = unit:getSpeed()
	
		local posx = tx - x
		local posy = ty - y
		local len = math.sqrt(posx*posx + posy*posy)
		local dx = posx/len * speed * dt
		local dy = posy/len * speed * dt
	
		if vector.dist(x,y, tx,ty) <= 2 then
			unit:setPos(math.floor(tx), math.floor(ty))
		else
			unit:setPos(x+dx, y+dy)
		end
	end
end

function unit:setImage(name)
	unit.image = love.graphics.newImage('assets/'..name..'.png')
end

function unit:isIdle()
	if unit:getPos() == unit:getTargetPos() then
		return true
	else
		return false
	end
end


function unit:attack(target)
	target:beAttacked(unit.damage)
end

function unit:beAttacked(damage)
	unit.health = unit.health - damage
	if unit.health <= 0 then
		unit.health = 0
		unit:die()
		entities.destroy(unit.id)
	end
end


function unit:load()
end

function unit:die()
end


return unit
