-- -- LICK allows realtime debugging
-- lick = require 'lib.lick'
-- lick.reset = true
-- lick.file = 'main.lua'

default = {}
default.width, default.height, default.fullscreen, default.vsync, default.fsaa = love.graphics.getMode()

-- HUMP Gamestates
gamestate = require 'lib.gamestate'
require 'ingame'
require 'menu'
currentGamestate = 'none'

-- HUMP Vector
vector = require 'lib.vector-light'

-- Quickie
gui = require 'lib.quickie'
-- Lazy font loading
-- fonts = setmetatable({}, {__index = function(t,k)
--     local f = love.graphics.newFont(k)
--     rawset(t, k, f)
--     return f
-- end })

font = love.graphics.newFont('assets/04b03.ttf', 16)

-- Handles entities
require 'entities'

function love.load()
	-- Set display
	modes = love.graphics.getModes()
	-- sort from smallest to largest
	table.sort(modes, function(a, b) return a.width*a.height < b.width*b.height end)
	
	-- Scale images using nearest neighbor to keep sharp edges
	love.graphics.setDefaultImageFilter('nearest', 'nearest')
	
	love.graphics.setFont(font)
	
	gamestate.registerEvents()
	gamestate.switch(menu)
end

function love.update(dt)
end

function love.draw()
	gui.core.draw()
	
	-- Print debug info
	love.graphics.print("FPS: "..love.timer.getFPS(), 0,0)
	love.graphics.print("Entities: "..entities.count, 0, 12)
	love.graphics.print("Gamestate: "..currentGamestate, 0,24)
end

function love.focus(bool)
end

function love.keypressed(key, unicode)
	gui.keyboard.pressed(key, unicode)
	if key == 'f' then
		local width, height, fullscreen, vsync, fsaa = love.graphics.getMode()
		if fullscreen == true then
			love.graphics.setMode(default.width, default.height, default.fullscreen, default.vsync, default.fsaa)
		else
			love.graphics.setMode(modes[#modes].width, modes[#modes].height, true)
		end
	end
end

function love.keyreleased(key, unicode)
end

function love.mousepressed(x,y, button)
end

function love.mousereleased(x,y, button)
end

function love.quit()
end
